#!/usr/bin/perl -w

# HTML.pm - The NagiosGrapher FE helper class.
# Copyright (C) 2004, NETWAYS GmbH, Gerd Mueller, Marius Hein
# $Id: HTML.pm 1625 2007-05-29 09:45:22Z gmueller $

#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# $Id: HTML.pm 1625 2007-05-29 09:45:22Z gmueller $
package NagiosGrapher::HTML;

no warnings qw/redefine/;

use strict;
use NagiosGrapher;

use File::Basename;
use CGI qw/:standard/;

use vars qw (
  @ISA
  @EXPORT
  @EXPORT_OK
  %EXPORT_TAGS
  $VERSION

  %ACTIONS
  %DEFAULTS
  %SITETITLES
  %REPLACEMARKS

  @FORM_HIDDENFIELDS

  %PAGE_REFRESH
  %RRD_TYPEVALUES

  $GRAPH_SCRIPT
);

our $VERSION = '1.7.1';
our @ISA = qw (Exporter);
our @EXPORT = qw(%ACTIONS %DEFAULTS %SITETITLES);

# The actions used by the CGI
our %ACTIONS = (
  'graph' => 'graphs an single graph or multigraph',
  'hostgraph' => 'graphing all services on a host',
  'rrdinfo' => 'displays the content of the rrddb',
  'showtemplate' => 'displays the template matching the criterias',
  'calendar' => 'shows the calendar and the AT-STYLE TIME SPECs',
);

our %DEFAULTS = (
  'refresh'	=> 180,
  'height'	=> 250,
  'width'	=> 720,
  'type'	=> 'AVERAGE',
);

our %SITETITLES = (
  'graph' => 'graph analysis for __HOST__, __SERVICE__',
  'hostgraph' => 'all graphs for __HOST__',
  'calendar' => 'calendar',
  'rrdinfo' => 'rrd info for __HOST__, __SERVICE__',
  'showtemplate' => 'templateinfo for __SERVICE__',
);

our %REPLACEMARKS = (
  '__HOST__' => 'host',
  '__SERVICE__' => 'service',
);

our @FORM_HIDDENFIELDS = ('browser', 'service', 'host', 'old_service', 'old_host');

our %RRD_TYPEVALUES = (
  'AVERAGE' => 'Average values',
  'MAX' => 'Max values',
  'MIN' => 'Min values',
);

our %PAGE_REFRESH = (
  "0"	=> "None",
  "60"	=> "1 minute",
  "180"	=> "3 minutes",
  "300"	=> "5 minutes",
  "600"	=> "10 minutes",
);


our $GRAPH_SCRIPT = './rrd2-system.cgi';
our $MENU_GRAPH_SCRIPT = './graphs.cgi';

sub new {
  my $this = shift;
  my $class = ref($this) || $this;

  my $NagiosGrapher = NagiosGrapher::getInstance();
  my $CGI = new CGI;

  my $self = bless {
    '_NagiosGrapher' => $NagiosGrapher,
    '_CGI' => $CGI,
    '_Script' => basename($0),

    '_SERVICE' => $CGI->param('service'),
    '_HOST' => $CGI->param('host'),

    '_FE_BROWSER_AUTH' => undef,

  }, $class;

  $self->SetParamDefaults();

  return $self;
}

sub SetParamDefaults {
  my $self = shift;
  while (my ($key, $value) = each(%NagiosGrapher::HTML::DEFAULTS)) {
    if ($self->Param($key) eq '') {
      $self->Param($key, $value);
    }
  }

  return 1;
}

sub NG {
  my $self = shift;
  return $self->{_NagiosGrapher} || undef;
}

sub Q {
  my $self = shift;
  return $self->{_CGI} || undef;
}

sub Service {
  my $self = shift;
  my ($service) = @_;

  # set default service as defined in main configuration if nothing has been passed
  unless ($self->Param('service')) {
    if ($self->NG) {
      $service = $self->NG->get_ngraphercfg_value('default_service') unless ($service);
    }
  }

  if ($service) {
    $self->{'_SERVICE'} = $service;
    $self->Param('service', $service);
  }

  return $self->{'_SERVICE'};
}

sub Host {
  my $self = shift;
  my ($host) = @_;

  # set default host as defined in main configuration if nothing has been passed
  unless ($self->Param('host')) {
    if ($self->NG) {
      $host = $self->NG->get_ngraphercfg_value('default_host') unless ($host);
    }
  }

  if ($host) {
    $self->{'_HOST'} = $host;
    $self->Param('host', $host);
  }

  return $self->{'_HOST'};
}

sub Param {
  my $self = shift;
  my ($param, $new) = @_;

  my %params = $self->Q->Vars;

  if ($new) {
    $self->Q->param($param, $new);
  }

  if (exists $params{$param} && length($params{$param}) > 0) {
    return $params{$param};
  }

  return undef;
}

sub getPages {
  my $self = shift;
  my ($service) = @_;

    my (%out);
    my @conf = $self->NG->get_ngrapherdef($service || $self->{'_SERVICE'} || $self->Param('service'));

    foreach (@conf) {
        if (exists $_->{page}) {
            $out{$_->{page}} = $_->{page}
        }
    }

    unless (keys %out > 0) {
      return;
      #return undef;
    }

    $out{"-ALL"} = '--ALL--';

    return %out;
}

sub getXMLHashWithMultigraph {
  my $self = shift;

  if ($self->NG) {
    my $xmlhash = $self->NG->get_ngrapherxml();

    my @nmgraphs = $self->NG->get_nmgrapherdef();
    if ($self->Host) {
      foreach (@nmgraphs) {
          unless ($xmlhash->{host}->{$_->{host_name}}) {
              $xmlhash->{host}->{$_->{host_name}}->{service}->{$_->{service_name}} = {};
          }
          else {
              unless ($xmlhash->{host}->{$_->{host_name}}->{service}->{$_->{service_name}}) {
                  $xmlhash->{host}->{$_->{host_name}}->{service}->{$_->{service_name}} = {};
              }
          }
      }
    }
    # Add the hostgraph servicename
    my $service = [sort(keys(%{$xmlhash->{host}->{$self->Host}->{service}}))]->[0];

    if ($self->Host && $xmlhash->{host}->{$self->Host}->{service}->{$service}->{rrd_file}) {
      unless ($xmlhash->{host}->{$self->Host}->{service}->{'* All Services'}) {
        $xmlhash->{host}->{$self->Host}->{service}->{'* All Services'} = {};
      }
    }

    return $xmlhash;
  }

  return undef;
}

sub replaceMarks {
  my $self = shift;
  my $text = shift;

  while (my($mark, $replace) = each(%NagiosGrapher::HTML::REPLACEMARKS)) {
    if ($text =~ m/$mark/g && $self->Param($replace)) {
      $text =~ s/$mark/$self->Param($replace)/ge;
    }
  }

  return $text;

}

sub getTitle {
  my $self = shift;
  my $action = $self->getAction();
  my $title = undef;

  if ($self->isAction($action) && exists $NagiosGrapher::HTML::SITETITLES{$action}) {
    $title = $self->replaceMarks(
      $NagiosGrapher::HTML::SITETITLES{$action}
    );
  }

  return $title;
}

sub SelfURL {
  my $self = shift;
  my %args = (
    'Type' => undef,
    @_
  );

  # we should not include protocol and host information, since these
  # information could be something else then visible from the outer world (e.g.
  # if there is sitting a proxy (SSL) in front of us
  my $url = $self->Q->url(-relative => 1, -query => 1);

  return $url unless($args{Type});

  if ($args{Type} eq 'old') {
    $url =~ s/;/&/g;
  }
  elsif ($args{Type} eq 'script') {
    $url = $self->{_Script};
  }

  return $url;
}

sub getAction {
  my $self = shift;

  my $action = lc( $self->Param('action') );
  $action = 'graph' unless ($action);

  if ($self->Service eq '* All Services') {
    $action = 'hostgraph';
  }

  return $action;
}

sub isAction {
  my $self = shift;

  my $check = $self->getAction();

  foreach (@_) {
    my $action = lc( $_ );
    if (exists $NagiosGrapher::HTML::ACTIONS{$check}
      && exists $NagiosGrapher::HTML::ACTIONS{$action}
      && $action eq $check ) {

      return 1;
    }
  }

  # default
  return undef;
}

sub checkUseTimefilter {
  my $self = shift;

  if ($self->NG) {
	return $self->NG->get_ngraphercfg_value_bool('fe_use_timefilter');
  }

  return undef;
}

sub checkPageBrowserAuth {
  my $self = shift;

  if ($self->NG && !defined $self->{'_FE_BROWSER_AUTH'}) {
    $self->{'_FE_BROWSER_AUTH'} = 0;

    # check auth all
    my $all = $self->NG->get_ngraphercfg_value_bool('fe_use_browser_all') || undef;

    # check url overwrite
    my $overwrite = $self->Param('browser');
    my $auth = $overwrite if ($overwrite && $self->NG->get_ngraphercfg_value_bool('fe_use_browser_url'));

    if ($auth) { # overwrite
      $self->{'_FE_BROWSER_AUTH'} = 1;
    }
    elsif ($self->NG->AuthCheckType($ENV{'REMOTE_USER'}, 'fe_use_browser_for')) { # user
      $self->{'_FE_BROWSER_AUTH'} = 1;
    }
    elsif ($all)  { # all
      $self->{'_FE_BROWSER_AUTH'} = 1;
    }
    else { # failed
      $self->{'_FE_BROWSER_AUTH'} = 0;
    }

    # rewrite the params for the browser ...
    if ($self->{'_FE_BROWSER_AUTH'} eq '1') {
      $self->Param('old_host', $self->Param('host'));
      $self->Param('old_service', $self->Param('service'));
    }

  }

  return $self->{'_FE_BROWSER_AUTH'};
}

sub htmlGraphImage {
  my $self = shift;
  my %args = (
    'Title' => undef,
    'Start' => undef,
    'End' => undef,
    'Host' => undef,
    'Service' => undef,
    @_
  );

  my (@args, @format_links);
  my @formats = qw(png pdf svg eps);

  push @args, sprintf('host=%s', $args{'Host'});
  push @args, sprintf('service=%s', $args{'Service'});
  push @args, sprintf('width=%d', $self->Param('width'));
  push @args, sprintf('height=%d', $self->Param('height'));

  if ($self->getPages() && $self->Param('page_act')) {
    push @args, sprintf('act_page=%s', $self->Param('page_act'));
  }
  push @args, sprintf('type=%s', $self->Param('type'));

  push @args, sprintf('title=%s', $args{'Title'}) if ($args{'Title'});
  push @args, sprintf('start=%s', $args{'Start'}) if ($args{'Start'});
  push @args, sprintf('end=%s', $args{'End'}) if ($args{'End'});

  my $zoomBox = undef;
  if ($args{'Title'} eq 'Custom') {
    $zoomBox = 1;
  }

  my $urlZoom = $NagiosGrapher::HTML::MENU_GRAPH_SCRIPT . '?' . join('&', @args);
  my $urlImg = $NagiosGrapher::HTML::GRAPH_SCRIPT . '?'. join(';', @args);

  foreach (@formats) {
    push @format_links, $self->Q->a({-href=>sprintf('%s;imageformat=%s', $urlImg, $_), -target=>'_blank'}, uc($_));
  }

  my $out = undef;
  $out .= '<tr><td width="45">&nbsp;</td><td>';
  $out .= '<table class="rrdheader"><tr>';
  $out .= '<td>[&nbsp;'. $self->Q->a({name=>$args{'Title'}, -href=>$self->SelfURL. '#top'}, 'To top');
  $out .= '&nbsp;]&nbsp;'. $args{'Title'}. ' graph'. '</td>';
  $out .= '<td align="right">View as:&nbsp;'. join(',&nbsp;', @format_links);
  $out .= '</td></tr></table>';
  if ($zoomBox) {
    $out .= '<div id="zoomBox" style="position:absolute; overflow:none; left:0px; top:0px; width:0px; height:0px; visibility:visible; background:green; filter:alpha(opacity=50); -moz-opacity:0.5; -khtml-opacity:0.5; opacity:0.5"></div>';
    $out .= '<div id="zoomBoxZone" style="position:absolute; overflow:none; left:0px; top:0px; width:0px; height:0px; visibility:visible; cursor:crosshair; background:blue; filter:alpha(opacity=0); -moz-opacity:0; -khtml-opacity:0; opacity:0" oncontextmenu="return false"></div>';
    $out .= $self->Q->img({src => '', alt => $args{Title}, id => 'ngGraph'});
  } else {
    $out .= $self->Q->a({href => $urlZoom}, $self->Q->img({src => $urlImg, alt => $args{Title}}));
  }
  $out .= '</td></tr>';
  $out .= q(<tr><td colspan="2" style="height: 1px; font-size: 1px;">&nbsp;</td></tr>);

  print $out;
}

sub htmlDefaultImageSelector {
  my $self = shift;

  print '<tr>',
    '<td>Width:</td><td>',
    $self->Q->textfield(-name=>"width", -default=>$self->Param('width'), -override=>1, -size=>4),
    '</td><td>Height:</td><td>',
    $self->Q->textfield(-name=>"height", -default=>$self->Param('height'), -override=>1, -size=>4),
    '</td><td>Refresh:</td><td>',
    $self->Q->popup_menu (
      -name		=> 'refresh',
      -default	=> $self->Param('refresh'),
      -values		=> [keys(%NagiosGrapher::HTML::PAGE_REFRESH)],
      -labels		=> \%NagiosGrapher::HTML::PAGE_REFRESH,
      -onchange	=> 'this.form.submit();'
    ),
    '</td></tr>';
}

sub htmlRRDTypeCombo {
  my $self = shift;

  print '<tr>',
  '<td>Type:</td><td colspan="5">',
  $self->Q->popup_menu (
    -name		=> 'type',
    -default	=> $self->Param('type'),
    -values		=> [keys(%NagiosGrapher::HTML::RRD_TYPEVALUES)],
    -labels		=> \%NagiosGrapher::HTML::RRD_TYPEVALUES,
    -style		=> 'width: 287px; background-color: #bbccff; color: #000000;',
    -onchange	=> 'this.form.submit();'
  ),
  '</td></tr>';
}

sub htmlTimeFilter {
  my $self = shift;

  if ($self->checkUseTimefilter()) {
    print q(<tr><td colspan="6" style="height: 1px; font-size: 1px;">&nbsp;</td></tr>);

    print '<tr><td>',
    $self->Q->a({
      href => '#',
      onclick => "javascript:window.open('". $self->SelfURL(Type => 'script'). "?action=calendar;field=fld-start', 'calendar', 'width=250,height=215');"
    }, 'Start'), ':</td><td colspan="6">',
    $self->Q->textfield (
      -name => 'start',
      -id => 'fld-start',
      -value => $self->Param('start'),
      -size=>18,
      -style=>"width: 287px;"
    ),
    '</td></tr>';

    print '<tr><td>',
    $self->Q->a({
      href => '#',
      onclick => "javascript:window.open('". $self->SelfURL(Type => 'script'). "?action=calendar;field=fld-stop', 'calendar', 'width=250,height=215');"
    }, 'End'), ':</td><td colspan="6">',
    $self->Q->textfield (
      -name => 'end',
      -id => 'fld-stop',
      -value => $self->Param('end'),
      -size=>18,
      -style=>"width: 287px;"
    ),
    '</td></tr>';

    if ($self->isAction('hostgraph')) {

      my %timeframes = (
        '' => '',
        '-5400' => 'Actual',
        '-129600' => 'Daily',
        '-907200' => 'Weekly',
        '-4017600' => 'Monthly',
        '-48211200' => 'Yearly',
      );

      print '<tr><td>Timeframe:</td><td colspan="6">',
      $self->Q->popup_menu (
        -name => 'timeframe',
        -value => $self->Param('timeframe'),
        -style =>"width: 287px;",
        -values => [sort keys(%timeframes)],
        -labels => \%timeframes,
		-onchange	=> 'this.form.submit();'
      ),
      '</td></tr>';

    }

    print q(<tr><td colspan="1">&nbsp;</td><td colspan="5">Only values defined by the <a target="_blank" href="http://oss.oetiker.ch/rrdtool/doc/rrdfetch.en.html">AT-STYLE TIME SPECIFICATION</a></td></tr>);
    print q(<tr><td colspan="6" style="height: 1px; font-size: 1px;">&nbsp;</td></tr>);
  }
}

sub htmlPageSelector {
  my $self = shift;

  my %page_hash = $self->getPages();

  if (keys (%page_hash) > 1) {

    my $page_act = $self->Param('page_act');
    $page_act=[keys(%page_hash)]->[1] unless ($page_act);
    $page_act=undef if ($page_act eq '-ALL');

    $self->Param('page_act', $page_act);

    print '<tr><td>page:</td><td colspan="5">',
      $self->Q->popup_menu(
        -name		=> 'page_act',
        -default	=> $page_act,
        -values		=> [keys(%page_hash)],
        -labels		=> \%page_hash,
        -style		=> 'width: 287px; background-color: #fc8888;',
        -onchange	=> 'this.form.submit();'
      ),
      '</td></tr>';
  }
  else {
    $self->Param('page_act', '');
  }
}

sub htmlHostServiceBrowser {
  my $self = shift;

  if ($self->checkPageBrowserAuth()) {
    my $xmlhash = $self->getXMLHashWithMultigraph();

    print '<tr><td>Host:</td><td colspan="5">',
    $self->Q->popup_menu (
      -name		=> 'host',
      -default	=> $self->Host() || undef,
      -values		=> [sort(keys(%{$xmlhash->{host}}))],
      -style		=> 'width: 287px; background-color: #ffff00; color: #000000;',
      -onchange	=> 'this.form.submit();'
    ), '</td></tr>';

    print '<tr><td>Service:</td><td colspan="5">',
    $self->Q->popup_menu (
      -name		=> 'service',
      -default	=> $self->Service() || undef,
      -values		=> [sort(keys(%{$xmlhash->{host}->{$self->Host}->{service}}))],
      -style		=> 'width: 287px; background-color: #ffff00; color: #000000;',
      -onchange	=> 'this.form.submit();'
    ), '</td></tr>';

  }

}

sub htmlHiddenFields {
  my $self = shift;
  my $out = undef;

  foreach my $name (@NagiosGrapher::HTML::FORM_HIDDENFIELDS) {
    my $default = $name;

    next if ($default =~ m/old_/ && !$self->checkPageBrowserAuth());
    next if (($default eq 'host' || $default eq 'service') && $self->checkPageBrowserAuth());

    $default =~ s/^old_//g;

    if ($self->Param($default)) {
      # reset if the fucking '-default' not works ...
      $self->Param($name, $self->Param($default));

      $out .= $self->Q->hidden (
        -name => $name,
        -default => $self->Param($default)
      );
    }

  }

  print $out;
}

sub htmlHeader {
  my $self = shift;

  $self->Q->default_dtd( '-//W3C//DTD HTML 4.01 Transitional//EN' );

  my $out = undef;
  my $meta = undef;

  if ($self->Param('refresh')) {
    $meta = {
      -http_equiv => 'refresh',
      -content => $self->Param('refresh'). '; URL='. $self->SelfURL(Type => 'old'),
    };
  }

  $out .= $self->Q->header(-type=>'text/html', -expires=>'+1s');
  $out .= $self->Q->start_html (
    -lang	=> 'de-DE',
    -title	=> $self->getTitle(),
    -style	=> { -code => $self->getCSS() },
    -script => { -code => $self->getZoomJavaScript() },
    -head	=> meta($meta),
  );

  $out .= $self->Q->a({-name=>'#top'}, '');

  print $out;
}

sub getCSS {
  # Default Stylesheet
  my $css =<<ECSS;
      body { font-family: 'Arial'; font-size: 8pt; color: #666666; background-color: #f1f1f1; }

      td { font-size: 8pt; color: #333333; }

      a { font-size: 8pt; text-decoration: none; color: #4d8fd2; font-weight: normal; }

      a:hover { text-decoration: underline; }

      .head2 { color: #333333; font-size: 8pt; font-weight: normal; }

      .head2 code { color: #cc0000; font-size: 8pt; font-weight: normal; }

      .head1 { color: #333333; font-size: 12pt; font-weight: bolder; }

      .table1 {
          margin-left: 10px;
          background-color: #ffffff;
          border-collapse: collapse;
          border-color: #f1f1f1;
      }

      .table1 td {
          border-color: #f1f1f1;
      }

      table.rrdheader {
        width: 100%;
        background-color: #f1f1f1;
        border-left: 2px #c0c0c0 solid;
        border-right: 2px #c0c0c0 solid;
        border-top: 2px #c0c0c0 solid;
      }

      table.rrdheader td { padding: 2px; }

      div.line { height: 1px; font-size: 1px; background-color: #000080; margin: 5px 0px 5px 0px; }
ECSS
  return $css;
}

sub getZoomJavaScript {
	# javascript for zooming feature
	my $js =<<EJS;
/*
 * ngZoom - provides zoom functionality for NagiosGrapher
 * Copyright (C) 2007 Christian DÃ¶bler <cdoebler_at_netways.de>
 *
 * inspired by and partially ripped from Bonsai - Copyright (C) 2004 Eric Steffen
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


/*
 * global variables
 */ 
var ngMouse;
var ngZoomBox;
var urlParams;

var nagiosPortal = false;
var nagiosPortalPrefix = 'tx_netsproxy_pi1';

var graphsCGI = 'graphs\.cgi';
var rrdCGI = 'rrd2-system.cgi';

var zoomBoxID = 'zoomBox';
var graphImageID = 'ngGraph';


/*
 * objects
 */
function mouseObj () {
	this.posX = 0;
	this.posY = 0;
	this.posXStart = 0;
	this.posYStart = 0;
	this.mouseWheelState = 0;
	this.drawBox = 0;
	this.graphStart = 0;
	this.graphEnd = 0;
	this.mouseButtonLeft = mouseButtonLeftFunc;
	this.mouseButtonRight = mouseButtonRightFunc;
	this.mouseButtonMiddle = mouseButtonMiddleFunc;
	this.mouseWheel = mouseWheelFunc;
	this.mouseGetPos = mouseGetPosFunc;
	this.getTimeFrame = mouseGetTimeFrameFunc;
	this.setEvent = mouseObjSetEvent;
}

function zoomBoxObj () {
	this.boxLeft = 0;
	this.boxRight = 0;
	this.boxWidth = 0;
	this.boxHeight = 0;
	this.drawBox = zoomBoxDrawFunc;
	this.refresh = zoomBoxRefresh;
	this.refresh();
}

function urlParamsObj () {
	this.url = false;
	this.param = new Array();
	this.paramNagiosPortal = new Array();
	this.getParams = getUrlParamsFunc;
	this.getParamsNagiosPortal = getUrlParamsNagiosPortalFunc;
	this.getParams();
}

function graphsObj () {
	this.id = Array();
	this.url = Array();
	this.params = Array();
	this.start = Array();
	this.end = Array();
}


/*
 * object functions
 */
function zoomBoxDrawFunc (x1, y1, x2, y2) {
	var divObj;
	var tmp;
	divObj = document.getElementById(zoomBoxID);
	if (ngMouse.drawBox) divObj.style.visibility = 'visible';
	else divObj.style.visibility = 'hidden';
	if (x2 < x1) { tmp = x2; x2 = x1; x1 = tmp; }
	if (y2 < y1) { tmp = y2; y2 = y1; y1 = tmp; }
	x1 -= this.boxLeft;
	x2 -= this.boxLeft;
	y1 -= this.boxTop;
	y2 -= this.boxTop;
	divObj.style.clip ="rect(" + y1 + "px " + x2 + "px " + y2 + "px " + x1 + "px)";
}

function zoomBoxRefresh () {
	var left = 0;
	var top = 0;
	var start = urlParams.param['start'];
	var end = urlParams.param['end'];
	if (ngMouse.graphStart) start = ngMouse.graphStart;
	if (ngMouse.graphEnd) end = ngMouse.graphEnd;
	document.getElementById(graphImageID).src = getGraphURL(start, end);
	divZoneObj = document.getElementById(zoomBoxID + 'Zone');
	imgObj = document.getElementById(graphImageID);
	divObj = document.getElementById(zoomBoxID);
	if (ngMouse.drawBox) divObj.style.visibility = 'visible';
	else divObj.style.visibility = 'hidden';
	this.boxWidth = urlParams.param['width'];
	this.boxHeight = urlParams.param['height'];
	do {
		left += imgObj.offsetLeft;
		top += imgObj.offsetTop;
		imgObj = imgObj.offsetParent;
	} while(imgObj);
	left += Math.round(this.boxWidth * .0944);
	top += Math.round(this.boxHeight * .13);
	this.boxTop = top;
	this.boxLeft = left;
	divObj.style.left = this.boxLeft + 'px';
	divObj.style.top = this.boxTop + 'px';
	divObj.style.width = this.boxWidth + 'px';
	divObj.style.height = this.boxHeight + 'px';
	divZoneObj.style.left = this.boxLeft + 'px';
	divZoneObj.style.top = this.boxTop + 'px';
	divZoneObj.style.width = this.boxWidth + 'px';
	divZoneObj.style.height = this.boxHeight + 'px';
}

function mouseButtonLeftFunc (e) {
	var buttonState = false;
	if (this.e.which) buttonState = (this.e.which == 1); // !IE
	else if (this.e.button) buttonState = (this.e.button == 1); // IE
	return buttonState;
}

function mouseButtonRightFunc (e) {
	var buttonState = false;
	if (this.e.which) buttonState = (this.e.which == 3); // !IE
	else if (this.e.button) buttonState = (this.e.button == 2); // IE
	return buttonState;
}

function mouseButtonMiddleFunc (e) {
	var buttonState = false;
	if (this.e.which) buttonState = (this.e.which == 2); // !IE
	else if (this.e.button) buttonState = (this.e.button == 4); // IE
	return buttonState;
}

function mouseWheelFunc (e) {
        var delta = 0;
        if (this.e.wheelDelta) { // IE/Opera
                delta = this.e.wheelDelta/120;
                if (window.opera) delta = -delta;
        } else if (this.e.detail) delta = -this.e.detail/3; // Mozilla
	if (delta > 0) this.mouseWheelState = 1;
	else if (delta < 0) this.mouseWheelState = -1;
}

function mouseGetPosFunc (e) {
	if (this.e.pageX || this.e.pageY) {
		this.posX = this.e.pageX;
		this.posY = this.e.pageY;
	} else if (this.e.clientX || this.e.clientY) {
		this.posX = this.e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
		this.posY = this.e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
	}
}

function mouseGetTimeFrameFunc () {
	var mouseX = 0;
	var start = 0;
	var end = 0;
	var boxStart = 0;
	var boxEnd = 0;

	if (this.graphEnd) {
		end = this.graphEnd;
	} else {
		if (urlParams.param['end']) {
			end = parseInt(urlParams.param['end']);
		} else {
			end = 0;
		}
	}

	if (this.graphStart) {
		start = this.graphStart;
	} else {
		start = parseInt(urlParams.param['start']);
		if (start < 0) {
			start = end + start;
		}
	}

	var secondsPerPx = (end - start) / parseInt(urlParams.param['width']);
	var endOfGraph = ngZoomBox.boxLeft + parseInt(urlParams.param['width']);

	if (this.posXStart < this.posX) {
		boxStart = this.posXStart;
		boxEnd = this.posX;
	} else {
		boxStart = this.posX;
		boxEnd = this.posXStart;
	}

	if (boxStart < ngZoomBox.boxLeft) boxStart = ngZoomBox.boxLeft;
	if (boxEnd > endOfGraph) boxEnd = endOfGraph;

	this.graphEnd = start + ((boxEnd - ngZoomBox.boxLeft) * secondsPerPx);
	this.graphStart = start + ((boxStart - ngZoomBox.boxLeft) * secondsPerPx);
}

function getUrlParamsFunc () {
	this.url = window.location.search.substring(0);
	var params = window.location.search.substring(1);
	if (params.length) {
		var paramsArr = params.split('&');
		for (var x = 0; x < paramsArr.length; x++) {
			var tmp = paramsArr[x].split('=');
			if (tmp[0] == 'start') {
				if (tmp[1] == '-21600' || tmp[1] == '-86400' || tmp[1] == '-604800' || tmp[1] == '-2419200' || tmp[1] == '-29030400') {
					tmp[1] = parseInt(tmp[1]) * 2;
				}
			}
			this.param[tmp[0]] = tmp[1];
		}
	}
}

function getUrlParamsNagiosPortalFunc () {
	var params = unescape(urlParams.param[sProxyParamName('query')]);
	var tmp1 = new Array();
	var tmp1 = params.split('&');
	var len = tmp1.length;
	if (len) {
		for (var x = 0; x < len; x++) {
			var tmp2 = tmp1[x].split('=');
			if (tmp2[0] == 'start') {
				if (tmp2[1] == '-21600' || tmp2[1] == '-86400' || tmp2[1] == '-604800' || tmp2[1] == '-2419200' || tmp2[1] == '-29030400') {
					tmp2[1] = parseInt(tmp2[1]) * 2;
				}
			}
			this.paramNagiosPortal[tmp2[0]] = tmp2[1];
			if (tmp2[0] == 'start' || tmp2[0] == 'end' || tmp2[0] == 'width' || tmp2[0] == 'height') {
				this.param[tmp2[0]] = tmp2[1];
			}
		}
	}
}


/*
 * event functions
 */
function onMouseDownEvent (e) {
	ngMouse.setEvent(e);
	if (ngMouse.mouseButtonLeft()) {
		if (ngMouse.posX > ngZoomBox.boxLeft && ngMouse.posX < ngZoomBox.boxLeft + ngZoomBox.boxWidth &&
			ngMouse.posY > ngZoomBox.boxTop && ngMouse.posY < ngZoomBox.boxTop + ngZoomBox.boxHeight) {
			ngMouse.drawBox = 1;
			ngMouse.posXStart = ngMouse.posX;
			ngMouse.posYStart = ngMouse.posY;
			ngZoomBox.drawBox(ngMouse.posX, ngMouse.posY, ngMouse.posX, ngMouse.posY);
		}
	}
}

function onMouseUpEvent (e) {
	ngMouse.setEvent(e);
	if (ngMouse.drawBox) {
		ngMouse.getTimeFrame();
		ngMouse.posXStart = 0;
		ngMouse.posYStart = 0;
		ngMouse.drawBox = 0;
		var start = ngMouse.graphStart;
		var end = ngMouse.graphEnd;
		ngMouse.graphEnd = 0;
		ngMouse.graphStart = 0;
		window.location.href = getGraphURL(start, end, true);
	}
}

function onMouseWheelEvent (e) {
	ngMouse.setEvent(e);
	ngMouse.mouseWheel();
}

function onMouseMoveEvent (e) {
	ngMouse.setEvent(e);
	ngMouse.mouseGetPos();
	if (ngMouse.drawBox) {
		ngZoomBox.drawBox(ngMouse.posXStart, ngMouse.posYStart, ngMouse.posX, ngMouse.posY);		
	}
}

function mouseObjSetEvent (e) {
	if (!e) this.e = window.event;
	else this.e = e;
}

function windowResizeEvent() {
	ngZoom.refresh();
}


/*
 * misc and helpers
 */
function isArray(testObject) {	 
	return testObject && !(testObject.propertyIsEnumerable('length')) && typeof testObject === 'object' && typeof testObject.length === 'number';
}

function getEndTime () {
	var now = new Date();
	var nowUTC = Date.UTC(now.getFullYear(), now.getMonth(), now.getDay(), now.getHours(), now.getMinutes(), now.getSeconds());;
	return nowUTC;
}

function qStrLen () {
	var h = window.location.href;
	var pos = h.indexOf('?');
	if (pos == -1) return 0;
	h = h.substr(pos);
	return h.length-1;
}

function getUrl () {
	var u=window.location.href;
	if (!qStrLen()) return u;
	var i = u.indexOf('?');
	return u.substr(0,i);
}

function getGraphURL (start, end, reloadPage) {
	var url = getUrl()+'?';
	var qString = '';
	qString = appendToQString(qString, 'host');
	qString = appendToQString(qString, 'service');
	qString = appendToQString(qString, 'width');
	qString = appendToQString(qString, 'height');
	qString = appendToQString(qString, 'type');
	qString = appendToQString(qString, 'title', 0, reloadPage);
	qString = appendToQString(qString, 'start', parseInt(start));
	qString = appendToQString(qString, 'end', parseInt(end));
	if (nagiosPortal) {
		var tmpQString = 'id=' + urlParams.param['id'];
		tmpQString += '&' + sProxyParam('host');
		tmpQString += '&' + sProxyParam('path');
		tmpQString += '&' + sProxyParam('scheme');
		tmpQString += '&' + sProxyParam('port');
		tmpQString += '&';
		qString = tmpQString + sProxyParamName('query') + '=' + escape(qString);
	}
	if (!reloadPage) {
		var regEx = eval('/'+graphsCGI+'/');
		if (nagiosPortal) {
			qString = qString.replace(regEx, rrdCGI);
		} else {
			url = url.replace(regEx, rrdCGI);
		}
	}
	return url + qString;
}

function appendToQString (qString, param, value, reloadPage) {
	var sep = '';
	var paramLocal = '';
	if (qString.length) sep = '&'
	if (nagiosPortal) {
		paramLocal = urlParams.paramNagiosPortal[param];
	} else {
		paramLocal = urlParams.param[param];
	}
	if (value) {
		qString += sep + param + '=' + value;
	} else {
		if (param == 'title' && (paramLocal == 'undefined' || reloadPage)) paramLocal = 'Custom';
		if (paramLocal) {
			qString += sep + param + '=' + paramLocal;
		}
	}
	return qString;
}

function checkNagiosPortal () {
	if (urlParams.param[sProxyParamName('query')]) {
		nagiosPortal = true;
		return true;
	}
	return false;
}

function sProxyParam (name) {
	return sProxyParamName(name) + '=' + urlParams.param[sProxyParamName(name)];
}

function sProxyParamName (name) {
	return nagiosPortalPrefix + '[' + name + ']';
}

/*
 * init
 */
function initNGZoom () {
	urlParams = new urlParamsObj();
	checkNagiosPortal();
	if (nagiosPortal) {
		urlParams.getParamsNagiosPortal();
	}
	if (urlParams.param['start'] || urlParams.param['end'] || nagiosPortal) {
		ngMouse = new mouseObj();
		ngZoomBox = new zoomBoxObj();
		ngGraphs = new graphsObj();
		document.onmousedown = onMouseDownEvent;
		document.onmouseup = onMouseUpEvent;
		document.onmousemove = onMouseMoveEvent;
		if (window.addEventListener) window.addEventListener('DOMMouseScroll', onMouseWheelEvent, false);
		window.onmousewheel = document.onmousewheel = onMouseWheelEvent;
		window.onresize = windowResizeEvent;
	}
}

window.onload = initNGZoom;
EJS
	return $js;
}

1;
